
/**
 * Core controller for widget aggregation.
 * @module
 */
define([],
    function() {
        /**
         * Widget controller constructor.
         * @class
         */
        return function(){
            // key: data type, value: widget list
            var mappings = {};
            var widgets = [];

            var preCallback = function(){};
            var postCallback = function(){};

            /**
             * add a widget to page
             * @return this
             */
            this.add = function(widget) {
                widgets.push(widget);

                if (widget.dataTypes) {
                    $.each(widget.dataTypes, function(index, dataType) {
                        if (!mappings[dataType])
                            mappings[dataType] = [];

                        mappings[dataType].push(widget);
                    });
                }
                
                return this;
            };

            /**
             * commit data and send an Ajax request, parse and deliver responded data to each widget
             * @param params page level parameters
             * @return this
             */
            this.commit = function(params) {
                // TODO, sending request to server side, deliver the result data to widgets according to mappings, and call this.render to finalize the widget visualization
                return this;
            };

            /**
             * render widget with data, exposing this method for easier local test
             * @param widget
             * @param data
             * @return this
             */
            this.render = function(widget, data) {
                widget.render.call(widget, data);
                return this;
            };

            /**
             * set page level callback when data retrieved, but before any widget rendering:
             * function(data) {} // returned value will be treated as the data updated for rendering
             * @param func
             * @return this
             */
            this.setPreCallback = function(func) {
                preCallback = func;
                return this;
            };

            /**
             * set page level post callback after the rendering of all widgets, specifically for customized interactions between widgets:
             * function(data) {}
             * @param func
             * @return this
             */
            this.setPostCallback = function(func) {
                postCallback = func;
                return this;
            };
        };
    }
);